//& ****************************************************************************
//&
//& Written by Ingo Blank, August 2004
//& Donated to the HP-GCC Development Team
//& Copyright (C) 2004 The HP-GCC Development Team
//&
//& ****************************************************************************
//&
//&
//& This file is part of HP-GCC.
//&
//& HP-GCC is free software; you can redistribute it and/or modify
//& it under the terms of the GNU General Public License as published by
//& the Free Software Foundation; either version 2, or (at your option)
//& any later version.
//& 
//& HP-GCC is distributed in the hope that it will be useful,
//& but WITHOUT ANY WARRANTY; without even the implied warranty of
//& MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//& GNU General Public License for more details.
//& 
//& You should have received a copy of the GNU General Public License
//& along with HP-GCC; see the file COPYING. 
//&
//& As a special exception, you may use this file as part of a free software
//& library without restriction.  Specifically, if other files instantiate
//& templates or use macros or inline functions from this file, or you compile
//& this file and link it with other files to produce an executable, this
//& file does not by itself cause the resulting executable to be covered by
//& the GNU General Public License.  This exception does not however
//& invalidate any other reasons why the executable file might be covered by
//& the GNU General Public License.
//&
//& ****************************************************************************


#ifndef _FPDEFS_H
#define _FPDEFS_H

// #define __IEEE_LITTLE_ENDIAN 1

#ifndef __IEEE_BIG_ENDIAN
#define __IEEE_BIG_ENDIAN 1
#endif

#ifndef __IEEE_BIG_ENDIAN
#ifndef __IEEE_LITTLE_ENDIAN
 #error Must define endianness
#endif
#endif

typedef unsigned int __uint32_t ;
typedef int __int32_t ;

/* A union which permits us to convert between a double and two 32 bit
   ints.  */

#ifdef __IEEE_BIG_ENDIAN


typedef union 
{
  double value;
  struct 
  {
    __uint32_t msw;
    __uint32_t lsw;
  } parts;
} ieee_double_shape_type;

#endif

#ifdef __IEEE_LITTLE_ENDIAN

typedef union 
{
  double value;
  struct 
  {
    __uint32_t lsw;
    __uint32_t msw;
  } parts;
} ieee_double_shape_type;

#endif

/* Get two 32 bit ints from a double.  */

#define EXTRACT_WORDS(ix0,ix1,d)                \
do {                                \
  ieee_double_shape_type ew_u;                    \
  ew_u.value = (d);                        \
  (ix0) = ew_u.parts.msw;                    \
  (ix1) = ew_u.parts.lsw;                    \
} while (0)

/* Get the more significant 32 bit int from a double.  */

#define GET_HIGH_WORD(i,d)                    \
do {                                \
  ieee_double_shape_type gh_u;                    \
  gh_u.value = (d);                        \
  (i) = gh_u.parts.msw;                        \
} while (0)

/* Get the less significant 32 bit int from a double.  */

#define GET_LOW_WORD(i,d)                    \
do {                                \
  ieee_double_shape_type gl_u;                    \
  gl_u.value = (d);                        \
  (i) = gl_u.parts.lsw;                        \
} while (0)

/* Set a double from two 32 bit ints.  */

#define INSERT_WORDS(d,ix0,ix1)                    \
do {                                \
  ieee_double_shape_type iw_u;                    \
  iw_u.parts.msw = (ix0);                    \
  iw_u.parts.lsw = (ix1);                    \
  (d) = iw_u.value;                        \
} while (0)

/* Set the more significant 32 bits of a double from an int.  */

#define SET_HIGH_WORD(d,v)                    \
do {                                \
  ieee_double_shape_type sh_u;                    \
  sh_u.value = (d);                        \
  sh_u.parts.msw = (v);                        \
  (d) = sh_u.value;                        \
} while (0)

/* Set the less significant 32 bits of a double from an int.  */

#define SET_LOW_WORD(d,v)                    \
do {                                \
  ieee_double_shape_type sl_u;                    \
  sl_u.value = (d);                        \
  sl_u.parts.lsw = (v);                        \
  (d) = sl_u.value;                        \
} while (0)

double __kernel_sin(double x, double y, int iy);
double __kernel_cos(double x, double y);
double __kernel_tan(double x, double y, int iy);
__int32_t __ieee754_rem_pio2(double x, double *y);
int __kernel_rem_pio2(double *x, double *y, int e0, int nx, int prec, const __int32_t *ipio2);



#include <hpmath.h>


#endif // _FPDEFS_H


