//& <prolog>
//& ****************************************************************************
//&
//& Written by Claudio Lapilli
//&
//& Copyright (C) 2005 HP-GCC Development Team
//&
//& ****************************************************************************
//&
//&
//& This file is part of HP-GCC.
//&
//& HP-GCC is free software; you can redistribute it and/or modify
//& it under the terms of the GNU General Public License as published by
//& the Free Software Foundation; either version 2, or (at your option)
//& any later version.
//& 
//& HP-GCC is distributed in the hope that it will be useful,
//& but WITHOUT ANY WARRANTY; without even the implied warranty of
//& MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//& GNU General Public License for more details.
//& 
//& You should have received a copy of the GNU General Public License
//& along with HP-GCC; see the file COPYING. 
//&
//& As a special exception, you may use this file as part of a free software
//& library without restriction.  Specifically, if other files instantiate
//& templates or use macros or inline functions from this file, or you compile
//& this file and link it with other files to produce an executable, this
//& file does not by itself cause the resulting executable to be covered by
//& the GNU General Public License.  This exception does not however
//& invalidate any other reasons why the executable file might be covered by
//& the GNU General Public License.
//&
//& ****************************************************************************

//& </prolog>

// $Header: /cvsroot/hpgcc/sources/hplib/saturn/sat_real2double.c,v 1.2 2005/05/19 09:01:58 iblank Exp $

#include <hpgcc49.h>





// LN(2)/LN(10) IN FIXED POINT, W/16 DIGITS FRACTIONAL PART
#define LN2_LN10  19728	// DIV BY 65536

unsigned long long sat_double2real(double num)
{
	union dblnumber {
		double dbl;
		struct { unsigned int hi,lo; };
	} number;

	if(num==0.0) return 0LL;

	number.dbl=num;
	int exp2=number.hi>>20;
	int exp10;
	int sign=exp2&0x800;
	exp2&=0x7ff;
	exp2-=1023+52;
	
	if(sign) number.dbl=-number.dbl;

	// GET EXPONENT IN BASE-10 TO EXTRACT
	exp10=((abs(exp2)*LN2_LN10)+32768)>>16;
	if(exp2>=0) {
		// USE 
		number.dbl/=i10powx(exp10);
		exp10=-exp10;
	}
	else {
		number.dbl*=i10powx(exp10);
	}

	// GET EXCEDENT EXPONENT IN BASE 2
	exp2=number.hi>>20;
	exp2&=0x7ff;
	exp2-=1023+52;

	unsigned long long mant=(((unsigned long long)((number.hi)|0x00100000)&0x001fffff)<<32);
	mant|= ((unsigned long long)number.lo);

	// NORMALIZE MANTISSA BASED ON THE EXCEDENT BASE-2 EXPONENT
	if(exp2>0) mant<<=exp2;
	else {
	mant*=10;
	++exp10;
	while(exp2<0) { mant=(mant+1)>>1; ++exp2; }	// ROUND LAST DIGIT BEFORE TRUNCATION
	}



	// HERE WE HAVE SIGN, INTEGER MANTISSA W/16 DECIMAL DIGITS (left-justified) + EXPONENT IN BASE-10

	// CONVERT MANTISSA TO 16-DIGIT BCD
	unsigned long long pow10_64[8];
	unsigned int pow10_32[8];
	unsigned int bcdhi=0,bcdlo=0,bcdmask;
	int f;
    
	pow10_32[0]=10;
	for(f=1;f<8;++f) {
		pow10_32[f]=pow10_32[f-1]*10;
	}

	for(f=0;f<8;++f) {
	pow10_64[f]=((unsigned long long)pow10_32[f])*100000000LL;
	}

	// ROUND HERE FOR 12-DIGIT REAL
	mant+=50000;
//  ROUND FOR 16-BIT EXTENDED MANTISSA
//	mant+=5;

	for(f=7,bcdmask=0x10000000;f>=0;--f) {
		while(mant>=pow10_64[f]) {
			mant-=pow10_64[f];
			bcdhi+=bcdmask;
		}
		bcdmask>>=4;
	}

	unsigned int mant32=(unsigned int)mant;
	for(f=7,bcdmask=0x10000000;f>=0;--f) {
		while(mant32>=pow10_32[f]) {
			mant32-=pow10_32[f];
			bcdlo+=bcdmask;
		}
		bcdmask>>=4;
	}

	// CORRECT EXPONENT
	exp10=-exp10+16;
	if(exp10<0) exp10+=1000;

	// ASSEMBLE CALCULATOR REAL

	mant=(((unsigned long long)bcdhi)<<32)+(unsigned long long)bcdlo;
	mant>>=4;
	if(sign) mant|=0x9000000000000000;

	mant&=~0xfff;

	bcdmask=0;
	while(exp10>=100) { bcdmask+=0x100; exp10-=100; }
	while(exp10>=10)  { bcdmask+=0x10; exp10-=10; }
	bcdmask+=exp10;
	mant|=bcdmask;

return mant;	
}