//@<head>
//& <prolog>
//& ****************************************************************************
//&
//& Written by Ingo Blank
//&
//& Copyright (C) 2006 Ingo Blank, HP-GCC Development Team
//&
//& ****************************************************************************
//&
//&
//& This file is part of HP-GCC.
//&
//& HP-GCC is free software; you can redistribute it and/or modify
//& it under the terms of the GNU General Public License as published by
//& the Free Software Foundation; either version 2, or (at your option)
//& any later version.
//& 
//& HP-GCC is distributed in the hope that it will be useful,
//& but WITHOUT ANY WARRANTY; without even the implied warranty of
//& MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//& GNU General Public License for more details.
//& 
//& You should have received a copy of the GNU General Public License
//& along with HP-GCC; see the file COPYING. 
//&
//& As a special exception, you may use this file as part of a free software
//& library without restriction.  Specifically, if other files instantiate
//& templates or use macros or inline functions from this file, or you compile
//& this file and link it with other files to produce an executable, this
//& file does not by itself cause the resulting executable to be covered by
//& the GNU General Public License.  This exception does not however
//& invalidate any other reasons why the executable file might be covered by
//& the GNU General Public License.
//&
//& ****************************************************************************

//& </prolog>

// $Header: /sources/hplib/stdio/_vsscanf.c,v 1.3 2006/01/16 19:27:25 iblank Exp $

#include <hpgcc49.h>
#include <stdarg.h>

//@</head>

/*************************************************************
 !!! CAUTION !!!
 
 Work in progress.
 This function isn't yet fully implemented feature wise.
 Also expect lots of bugs!
 
 *************************************************************/

// TODO: Add missing features, unit testing

#define WHITESPACE " \t\r\n"

//@<func name="_vsscanf">

int
_vsscanf(const char *str, const char *format, va_list ap)
{
    int items = 0;
    int chars = 0;
    char *flag_chars = "ahlLq";
    char *conv_chars = "%dDiouxXfegEsc[pn";
    
    const char *p;
    
    for (p = format; *p ; p++) {
        
        if (*p != '%') {
            
            if(strchr(WHITESPACE,*p)) {
                while (strchr(WHITESPACE,*str)) {
                    str++;
                    chars++;
                }
                continue;
            }
            
            else if (*p == *str) {
                chars++;
                str++;
                continue;
            }
            
            else goto stop;
        }
        
        // cond: *p == '%' 

        p++;
        
        
        int star,width;
        char flag,conv;
        
        width = 0;
        
        // first check for possible flags ...
        
        if((star = *p == '*'))
            p++;
        
        if((flag = strchr(flag_chars,*p) ? *p : '\0'))
            p++;
        
        
        // check for 'll'
        
        if (flag == 'l' && *p == 'l') {
            flag = 'L'; p++;
        }
        
        if (flag == 'q')
            flag = 'L';
        
        // now for a possible field width ...
        
        if (isdigit(*p))
            for ( ; isdigit(*p) ; p++)
                width = 10*width + (*p & 0xF);
        
            
        // get conversion char & check validity ...

        if (! strchr(conv_chars,conv = *p))
            goto stop;
        
        // set default width, if necessary
        
        if (!width)
            width = conv == 'c' ? 1 : 0x7fffffff;
        
        int skip_to_delim = 1;
        static char skip_buff[2];
        
        char *skip_set = strchr(WHITESPACE,(skip_buff[0]=*(p+1))) ? WHITESPACE : skip_buff ;
        
        switch (conv) {
            
            int base;
            
            case '%':
                if (*str != '%') goto stop;
                break;
                
            case 'f':
            case 'e':
            case 'g':
            case 'E':
                
                if (! star) {
                    double *dval = va_arg(ap,double *);
                    *dval = atof(str);
                    items++;
                }
                
                break;
                
            
            case 'd':
            case 'D':

                if (!star) {
                 
                    if (flag == 'L') {
                        LONGLONG *ll = va_arg(ap,LONGLONG *);
                        *ll = atoll(str);
                    }
                    else if (flag == 'h') {
                        short *ii = va_arg(ap,short *);
                        *ii = (short)(atoi(str) & 0xFFFF);
                    }
                    else {
                        int * ii = va_arg(ap,int *);
                        *ii = atoi(str);
                    }
                    
                    items++;
                }
                
                break;
                
            
            case 'u':
            case 'x':
            case 'X':
            case 'o':
                 
            base = conv == 'u' ? 10 : (conv == 'o' ? 8 : 16);
                   
            if (!star) {
                 
                    if (flag == 'L') {
                        ULONGLONG *ll = va_arg(ap,ULONGLONG *);
                        *ll = atoull_base(str,base);
                    }
                    else if (flag == 'h') {
                        unsigned short *ii = va_arg(ap, unsigned short *);
                        *ii = (unsigned short)(atoui_base(str,base) & 0xFFFF);
                    }
                    else {
                        unsigned int * ii = va_arg(ap,unsigned int *);
                        *ii = atoui_base(str,base);
                    }
                    
                    items++;
                }
                
                break;
                
    
            case 'i':
            case 'p':
            
                if (! star) {
                
                    int minus;
                    char *u = str;
                    
                    base = 10;
                    
                    if ((minus = *u == '-'))
                        u++;
                    
                    if (*u == '0') {
                        u++;
                        if (*u == 'x' || *u == 'X') {
                            u++;
                            base = 16;
                        }
                        else
                            base = 8;
                    }
                    
                    // cond: u points to first significant digit
                    
                    if (conv == 'p') {
                        
                        void **p = va_arg(ap,void **);
                        *p = (void *) atoi_base(u,base);
                    }
                    else    
                        
                        if (flag == 'L') {
                            LONGLONG * ii = va_arg(ap,LONGLONG *);
                            LONGLONG z = atoll_base(u,base);
                            *ii = minus ? -z : z;
                        }
                        else if (flag == 'h') {
                            short *ii = va_arg(ap,short *);
                            short z = (short)(atoi_base(u,base) & 0xFFFF);
                            *ii = minus ? -z : z;
                        }
                        else {
                            int *ii = va_arg(ap, int *);
                            int z = atoi_base(u,base);
                            *ii = minus ? -z : z;
                        }
                    
                    items++;
                }
            
            break;
            

            case 's':
            case 'c':
                
            {

                const char *u = str;
                int c = 0;
                
                skip_to_delim = conv == 's';
                
                while (*u && c < width) {
                    if (conv == 's' && strchr(WHITESPACE,*u))
                        break;
                    u++;
                    c++;
                }
                
                if (! star ) {
                    
                    char *t;
                    
                    if (flag == 'a') {
                        char **sval = va_arg(ap,char **);
                        t = *sval = (char *) malloc(c + 1);
                    }
                    else 
                        t = va_arg(ap,char *);
                    
                    strncpy(t,str,c);
                    
                    if ((conv == 's') || ((conv == 'c') && (flag == 'a')))
                        *(char *)((unsigned long) t + c) = '\0';
                    
                    
                    items++;
                    
                }
                
            }
            
            break;
            
            case 'n':
                // return # of chars read so far (metadata conversion)
                {
                    int *ival = va_arg(ap,int *);
                    *ival = chars;
                    
                }
            break;
                
        }
        
        // now skip chars in source string up to matching delimiter(s) ...    
     
        if (skip_to_delim) {
            while(*str && !strchr(skip_set,*str)) {
                chars++ ; str++;
            }
         
        }
            
        else {
            chars += width;
            str = (char *) ((unsigned long) str + width);
        }    
                
    
    }
    
stop:

    return items;
}

//@</func>

